package SQ.Blocks.Indicators.ATRPercent;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Blocks.Indicators.MTATR.MTATR;
import SQ.Blocks.Indicators.ATR.ATR;
import com.strategyquant.tradinglib.simulator.Engines;
import SQ.Internal.IndicatorBlock;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(ATRP) ATR Percent", display="ATR Percent(@Chart@#ATRPeriod#)[#Shift#]", returnType = ReturnTypes.Number)
@Indicator(oscillator=false,min=0, max=1, step=0.01)
@Help("ATRPercent help text")
@ParameterSet(set="ATRPeriod=12")
@ParameterSet(set="ATRPeriod=24")
@ParameterSet(set="ATRPeriod=48")
@ParameterSet(set="ATRPeriod=10")
@ParameterSet(set="ATRPeriod=20")
@ParameterSet(set="ATRPeriod=40")
public class ATRPercent extends IndicatorBlock {

	@Parameter
	public ChartData Input;

	@Parameter(defaultValue="12", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int ATRPeriod;

	@Output
	public DataSeries Value;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	/**
	 * This method is called on every bar update and here the indicator value is computed.
	 *
	 * Unlike in MT4 you don't compute indicator values for multiple bars in a loop, 
	 * you need to compute value only for the latest (current) bar.
	 * Trading engine will take care of calling this method for every bar in the chart.
	 *
	 * Actual bar for which the indicator value is computed is stored in CurrentBar variable. 
	 * If 0, it means it is the very first bar of the chart.
	 */

	@Override
	protected void OnBarUpdate() throws TradingException {
		if(Engines.isTradestationEngine(Indicators.Engine)) {
			onBarUpdateTS();
		}
		else {
			
			onBarUpdateMT();
		}
	}

	protected void onBarUpdateMT() throws TradingException {

		// MT4/5 ATR
		MTATR indicator = Indicators.MTATR(Input, ATRPeriod);
		double atrValue = indicator.Value.getRounded(0,5);
		double ATRP  = (atrValue/Input.Close.get(0))*100;
		Value.set(0,ATRP);
		}	

	protected void onBarUpdateTS() throws TradingException {
			
		ATR indicator = Indicators.ATR(Input, ATRPeriod);
		double atrValue = indicator.Value.get(0);
		double ATRP  = SQUtils.round((atrValue/Input.Close.get(0))*100,5);
		Value.set(0,ATRP);
    }

}