package SQ.Blocks.Indicators.DEMA;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Calculators.*;

import SQ.Internal.IndicatorBlock;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(DEMA) DEMA", display="DEMA(@Chart@#Period#,#AppliedPrice#)[#Shift#]", returnType = ReturnTypes.Price)
@Help("Double Exponential Moving Average ")
@ParameterSet(set="Period=10")
@ParameterSet(set="Period=20")
@ParameterSet(set="Period=40")
@ParameterSet(set="Period=60")
@ParameterSet(set="Period=12")
@ParameterSet(set="Period=24")
@ParameterSet(set="Period=48")
@ParameterSet(set="Period=120")
public class DEMA extends IndicatorBlock {

	@Parameter
	public ChartData Chart;

	@Parameter(defaultValue="12", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int Period;

	@Parameter(defaultValue="0")
	@Editor(type=Editors.Selection, values="Close=0,Open=1,High=2,Low=3")
	public int AppliedPrice;

	@Output
	public DataSeries Value;


	
	private AverageCalculator openCalculator;
	private AverageCalculator highCalculator;
	private AverageCalculator lowCalculator;
	private AverageCalculator closeCalculator;
	
	private AverageCalculator DEMAOpenCalculator;
	private AverageCalculator DEMAHighCalculator;
	private AverageCalculator DEMALowCalculator;
	private AverageCalculator DEMACloseCalculator;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	@Override
	protected void OnInit() throws TradingException {
		
		openCalculator = new AverageCalculator(AverageCalculator.EMA, Period);
		highCalculator = new AverageCalculator(AverageCalculator.EMA, Period);
		lowCalculator = new AverageCalculator(AverageCalculator.EMA, Period);
		closeCalculator = new AverageCalculator(AverageCalculator.EMA, Period);

		DEMAOpenCalculator = new AverageCalculator(AverageCalculator.EMA, Period);
		DEMAHighCalculator = new AverageCalculator(AverageCalculator.EMA, Period);
		DEMALowCalculator = new AverageCalculator(AverageCalculator.EMA, Period);
		DEMACloseCalculator = new AverageCalculator(AverageCalculator.EMA, Period);

	}
	@Override
	protected void OnBarUpdate() throws TradingException {
      double doubleEMA = 0;

		if (CurrentBar < Period) {
			// what to set when there are less bars than indicator period parameter
			 Value.set(0, 0);

		} else {
			/// Finally the indicator DEMA = 2 * MME1 – MME2
			/// Double EMA = 2 x EMA – EMA (EMA)
			if(AppliedPrice ==0){

				closeCalculator.onBarUpdate(Chart.Close.get(0), CurrentBar);
				double EMAcClose = closeCalculator.getValue();
				DEMACloseCalculator.onBarUpdate(EMAcClose, CurrentBar);
				double DEMAcClose = DEMACloseCalculator.getValue();
				doubleEMA = 2* EMAcClose - DEMAcClose;
			}
			else if(AppliedPrice ==1){

				openCalculator.onBarUpdate(Chart.Open.get(0), CurrentBar);
				double EMAcOpen = openCalculator.getValue();
				DEMAOpenCalculator.onBarUpdate(EMAcOpen, CurrentBar);
				double DEMAcOpen = DEMAOpenCalculator.getValue();
				doubleEMA = 2* EMAcOpen - DEMAcOpen;
			}
			else if(AppliedPrice ==2){

				highCalculator.onBarUpdate(Chart.High.get(0), CurrentBar);
				double EMAcHigh = highCalculator.getValue();
				DEMAHighCalculator.onBarUpdate(EMAcHigh, CurrentBar);
				double DEMAcHigh = DEMAHighCalculator.getValue();
				doubleEMA = 2* EMAcHigh - DEMAcHigh;
			}
			else if(AppliedPrice ==3){

				lowCalculator.onBarUpdate(Chart.Low.get(0), CurrentBar);
				double EMAcLow = lowCalculator.getValue();
				DEMALowCalculator.onBarUpdate(EMAcLow, CurrentBar);
				double DEMAcLow = DEMALowCalculator.getValue();
				doubleEMA = 2* EMAcLow - DEMAcLow;
			}

		}

	  Value.set(0, doubleEMA);
      
	}
}