package SQ.Blocks.Indicators.DEMA;

import SQ.Internal.ConditionBlock;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;

@BuildingBlock(name="(DDR) Fast DEMA is Above Slow DEMA and both of them are rising", display="Fast DEMA(@Chart@#FastDEMAPeriod#,#AppliedPrice#) is above Slow DEMA(#SlowDEMAPeriod#,#AppliedPrice#) and both of them are rising", returnType = ReturnTypes.Boolean)
@Help("Is triggered if Fast DEMA is Above Slow DEMA and both of them are rising")
@OppositeBlock("DoubleDEMAFalling")
@MT5ExtendedTemplate
@ParameterSet(set="FastDEMAPeriod=10,SlowDEMAPeriod=20")
@ParameterSet(set="FastDEMAPeriod=20,SlowDEMAPeriod=40")
@ParameterSet(set="FastDEMAPeriod=40,SlowDEMAPeriod=80")
@ParameterSet(set="FastDEMAPeriod=60,SlowDEMAPeriod=120")
@ParameterSet(set="FastDEMAPeriod=12,SlowDEMAPeriod=24")
@ParameterSet(set="FastDEMAPeriod=24,SlowDEMAPeriod=48")
@ParameterSet(set="FastDEMAPeriod=48,SlowDEMAPeriod=96")
@ParameterSet(set="FastDEMAPeriod=120,SlowDEMAPeriod=240")
public class DoubleDEMARising extends ConditionBlock {
	
	@Parameter
	public ChartData Chart;
	
	@Parameter(defaultValue="12", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int FastDEMAPeriod;

	@Parameter(defaultValue="24", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int SlowDEMAPeriod;

	@Parameter(defaultValue="0")
	@Editor(type=Editors.Selection, values="Close=0,Open=1,High=2,Low=3")
	public int AppliedPrice;

	@Parameter
	public int Shift;
	
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	
	@Override
	public boolean OnBlockEvaluate() throws TradingException {
		DEMA fastIndicator = Strategy.Indicators.DEMA(Chart, FastDEMAPeriod,AppliedPrice);
		DEMA slowIndicator = Strategy.Indicators.DEMA(Chart, SlowDEMAPeriod,AppliedPrice);
		double fastDEMACurr= fastIndicator.Value.getRounded(Shift );
		double fastDEMAPrev = fastIndicator.Value.getRounded(Shift + 1);
		double slowDEMACurr= slowIndicator.Value.getRounded(Shift );
		double slowDEMAPrev = slowIndicator.Value.getRounded(Shift + 1);
		
		return (fastDEMACurr > slowDEMACurr) && (fastDEMACurr > fastDEMAPrev) && (slowDEMACurr > slowDEMAPrev);
	}

}