package SQ.Blocks.Indicators.DVO;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Calculators.AverageCalculator;
import java.lang.Math.*;

import SQ.Internal.IndicatorBlock;


@BuildingBlock(name="(DVO) David Varadi Oscillator", display="David Varadi Oscillator (DVO) (@Chart@#MAPeriod#,#PercentRankPeriod#)[#Shift#]", returnType = ReturnTypes.Number)
@Indicator(oscillator=true, middleValue=50, min=0, max=100, step=0.01)
@ParameterSet(set="MAPeriod=10,PercentRankPeriod=20")
@ParameterSet(set="MAPeriod=40,PercentRankPeriod=80")
@ParameterSet(set="MAPeriod=80,PercentRankPeriod=160")
@ParameterSet(set="MAPeriod=12,PercentRankPeriod=24")
@ParameterSet(set="MAPeriod=24,PercentRankPeriod=120")
@ParameterSet(set="MAPeriod=120,PercentRankPeriod=240")
@Help("DVO help text")
public class DVO extends IndicatorBlock {

	@Parameter(defaultChartIndex=0)
	public ChartData Chart;

	@Parameter(defaultValue="24", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int MAPeriod;

	@Parameter(defaultValue="120", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int PercentRankPeriod;

	@Output(name = "DVO", color = Colors.Red)
	public DataSeries Value;

	@Buffer
	public DataSeries Buffer;

	private AverageCalculator SMAAverageCalculator;
	//private AverageCalculator EMAAverageCalculator;
	//private AverageCalculator WMAAverageCalculator;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	protected void OnInit() throws TradingException {

		SMAAverageCalculator = new AverageCalculator(AverageCalculator.SMA, MAPeriod);

	}

	@Override
	protected void OnBarUpdate() throws TradingException {
      double price =0 ;
	  double dvo = 0;
	  int pRank =0;
	  double maValue =0;
	  

		if (CurrentBar < Math.max(PercentRankPeriod,MAPeriod)) {
			
			Value.set(0, 0);
			Buffer.set(0,0);

		} else {

			SMAAverageCalculator.onBarUpdate(Chart.Close.get(0)/((Chart.High.get(0)+Chart.Low.get(0))/2), CurrentBar);
			maValue = SMAAverageCalculator.getValue();
					
			Buffer.set(0,maValue);
	
		if(CurrentBar<PercentRankPeriod+MAPeriod+1){
			Value.set(0, 0);
		}
		else{
			
			for(int i = 1; i<= PercentRankPeriod; i++){
			if(Buffer.getRounded(0,6)>Buffer.getRounded(i,6)){

				pRank = pRank+1;
				}
			}
		}

		dvo = 100.0d*pRank/(double)PercentRankPeriod;
			
		}

	Value.set(0,dvo);
	  
	}
}