package SQ.Blocks.Indicators.DidiIndex;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Calculators.AverageCalculator;

import SQ.Internal.IndicatorBlock;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(DDI) Didi Index", display="Didi Index(@Chart@#FastPeriod#,#MediumPeriod#,#SlowPeriod#).#Line#[#Shift#]", returnType = ReturnTypes.Number)
@Indicator(oscillator=true, middleValue=1, min=0.9, max=1.1, step=0.001)
public class DidiIndex extends IndicatorBlock {

	@Parameter
	public ChartData Chart;

	@Parameter(defaultValue="3", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int FastPeriod;

	@Parameter(defaultValue="8", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int MediumPeriod;

	@Parameter(defaultValue="20", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int SlowPeriod;

	@Parameter(name="MAMethod", defaultValue="0")
	@Editor(type=Editors.Selection, values="Simple=0,Exponential=1,Smoothed=2,Linear weighted=3")
	public int MAMethod;

	@Output(name="FastValue", color=Colors.Yellow)
	public DataSeries FastValue;

	@Output(name="MedValue", color=Colors.Red)
	public DataSeries MedValue;

	@Output(name="SlowValue", color=Colors.Green)
	public DataSeries SlowValue;

	private AverageCalculator fastAverageCalculator;
	private AverageCalculator mediumAverageCalculator;
	private AverageCalculator slowAverageCalculator;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

@Override
	protected void OnInit() throws TradingException {
		
		MAMethod = SQUtils.fixAllowedRange(MAMethod, 0, 3, 0);
		
		fastAverageCalculator = new AverageCalculator(MAMethod, FastPeriod);
		mediumAverageCalculator = new AverageCalculator(MAMethod, MediumPeriod);
		slowAverageCalculator = new AverageCalculator(MAMethod, SlowPeriod);
	}


	@Override
	protected void OnBarUpdate() throws TradingException {
      double indiValue;

		if (CurrentBar < Math.max(FastPeriod,Math.max(MediumPeriod,SlowPeriod))) {
			// what to set when there are less bars than indicator period parameter
			FastValue.set(0, 0);
			SlowValue.set(0, 0);

		} else {

			fastAverageCalculator.onBarUpdate(Chart.Close.get(0), CurrentBar);
			mediumAverageCalculator.onBarUpdate(Chart.Close.get(0), CurrentBar);
			slowAverageCalculator.onBarUpdate(Chart.Close.get(0), CurrentBar);

			double fMa = fastAverageCalculator.getValue();
			double mMa = mediumAverageCalculator.getValue();
			double sMa = slowAverageCalculator.getValue();

			FastValue.set(0, fMa/mMa);
			SlowValue.set(0, sMa/mMa);
			MedValue.set(0,1);
		
		}

      // set indicator value to a Value output array.
      // The array is zero based, index 0 means it is value at current bar.
      
	}
}