package SQ.Blocks.Indicators.DisparityIndex;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Calculators.AverageCalculator;

import SQ.Internal.IndicatorBlock;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(DIX) Disparity Index", display="Disparity Index(@Chart@#Period#)[#Shift#]", returnType = ReturnTypes.Number)
@Indicator(oscillator=true, middleValue=0, min=-100, max=100, step=0.001)
public class DisparityIndex extends IndicatorBlock {

	@Parameter(defaultChartIndex=0)
	public ChartData Chart;

	@Parameter(defaultValue="10", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int Period;

	@Output
	public DataSeries Value;

	private AverageCalculator EmaAverageCalculator;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

protected void OnInit() throws TradingException {

		EmaAverageCalculator = new AverageCalculator(AverageCalculator.EMA, Period);

	}
	@Override
	protected void OnBarUpdate() throws TradingException {
      double indyValue = 0;

		if (CurrentBar < Period) {
			// what to set when there are less bars than indicator period parameter
			Value.set(0,0);

		} else {

			EmaAverageCalculator.onBarUpdate(Chart.Close.get(0), CurrentBar);

			double maValue = EmaAverageCalculator.getValue();
			
			indyValue = SQUtils.safeDivide((Chart.Close.get(0)-maValue), maValue)*100;	
			
		}

		Value.set(0,indyValue);
      
	}
}