package SQ.Blocks.Indicators.EhlersHilbertTransform;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import com.strategyquant.tradinglib.simulator.Engines;

import SQ.Internal.IndicatorBlock;

@BuildingBlock(name="(EHT) Ehlers Hilbert Transform", display="Ehlers Hilbert Transform(@Chart@).#Line#[#Shift#]", returnType = ReturnTypes.Number)
@Indicator(oscillator=true, middleValue=0)
@Help("EhlersHilbertTransform help text")
public class EhlersHilbertTransform extends IndicatorBlock {

	@Parameter(defaultChartIndex=0)
	public ChartData Chart;

	@Output(name="InPhase", color=Colors.Red)
	public DataSeries InPhase;

	@Output(name="Quadrature", color=Colors.Green)
	public DataSeries Quadrature;

	@Buffer
	public DataSeries Smooth;

	@Buffer
	public DataSeries Detrender;

	@Buffer
	public DataSeries Per;

	@Buffer
	public DataSeries Im;

	@Buffer
	public DataSeries Re;

	@Buffer
	public DataSeries SmoothPeriod;


	@Buffer
	public DataSeries Q1;


	@Buffer
	public DataSeries I1;


	@Buffer
	public DataSeries jI;


	@Buffer
	public DataSeries jQ;


	@Buffer
	public DataSeries I2;
	
	@Buffer
	public DataSeries Q2;


	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	   @Override
		protected void OnBarUpdate() throws TradingException {
		if(Engines.isTradestationEngine(Indicators.Engine)) {
			onBarUpdateTS();
		}
		else {
			
			onBarUpdateMT();
		}
	}

	
	protected void onBarUpdateMT() throws TradingException {

  		if (CurrentBar < 100) {

			// what to set when there are less bars than indicator period parameter
			InPhase.set(0, 0);
			Quadrature.set(0, 0);
			Smooth.set(0, 0);
			Detrender.set(0, 0);
			I1.set(0, 0);
			Q1.set(0, 0);
			jI.set(0, 0);
			jQ.set(0, 0);
			I2.set(0, 0);
			Q2.set(0, 0);
			Re.set(0, 0);
			Per.set(0, 0);
			SmoothPeriod.set(0, 0);

		} else {


			double p0 = (Chart.High.get(0) + Chart.Low.get(0)) / 2;
			double p1 = (Chart.High.get(1) + Chart.Low.get(1)) / 2;
			double p2 = (Chart.High.get(2) + Chart.Low.get(2)) / 2;
			double p3 = (Chart.High.get(3) + Chart.Low.get(3)) / 2;  

			Smooth.set(0, (4 * p0 + 3 * p1 + 2 * p2 + p3) / 10);

			double detrender = (0.0962 * Smooth.get(0) + 0.5769 * Smooth.get(2) - 0.5769 * Smooth.get(4) - 0.0962 * Smooth.get(6)) * (0.075  + 0.54);
			Detrender.set(0,detrender);

			double q1 = (0.0962 * Detrender.get(0) + 0.5769 * Detrender.get(2)  -  0.5769 * Detrender.get(4) - 0.0962 * Detrender.get(6)) * (0.075  + 0.54);	
			Q1.set(0,q1); 
			I1.set(0,Detrender.get(3)); 

			double ji = (0.0962 * I1.get(0) + 0.5769 * I1.get(2) - 0.5769 * I1.get(4) - 0.0962 * I1.get(6)) * (0.075  + 0.54); 	
			jI.set(0,ji);
			
			double jq = (0.0962 * Q1.get(0) + 0.5769 * Q1.get(2) - 0.5769 * Q1.get(4) - 0.0962 * Q1.get(6)) * (0.075  + 0.54); 	
			jQ.set(0,jq);		
		
			I2.set(0,I1.get(0) - jQ.get(0)); 
			Q2.set(0,Q1.get(0) + jI.get(0));
			
			Re.set(0,I2.get(0) * I2.get(0)    + Q2.get(0) * Q2.get(1));
			Im.set(0,I2.get(0) * Q2.get(1)  - Q2.get(0) * I2.get(1));

			Re.set(0,SQUtils.round(0.2   * Re.get(0)      + 0.8   * Re.get(1),8));
			Im.set(0,SQUtils.round(0.2   * Im.get(0)      + 0.8   * Im.get(1),8));

			if(Im.get(0)!=0 && Re.get(0)!=0){

				double per = 2 * Math.PI  / (Math.atan(Im.get(0) / Re.get(0)) );
				Per.set(0,per); 

			}
			else Per.set(0,0); 

			
			if(Per.get(0) > 1.5  * Per.get(1)) Per.set(0,1.5  * Per.get(1)); 			 
			if(Per.get(0) < 0.67  * Per.get(1)) Per.set(0,0.67  * Per.get(1)); 
			if(Per.get(0) < 6)  Per.set(0,6);
			if(Per.get(0) > 50)  Per.set(0,50);

			Per.set(0,0.2  * Per.get(0) + 0.8  * Per.get(1)); 
			SmoothPeriod.set(0, 0.33 * Per.get(0) + 0.67 * SmoothPeriod.get(1)); 	
					
			InPhase.set(0,I1.get(0)); 
			Quadrature.set(0,Q1.get(0));	

		}		
	}

	protected void onBarUpdateTS() throws TradingException {

		  	if (CurrentBar < 100) {

			// what to set when there are less bars than indicator period parameter
			InPhase.set(0, 0);
			Quadrature.set(0, 0);
			Smooth.set(0, 0);
			Detrender.set(0, 0);
			I1.set(0, 0);
			Q1.set(0, 0);
			jI.set(0, 0);
			jQ.set(0, 0);
			I2.set(0, 0);
			Q2.set(0, 0);
			Re.set(0, 0);
			Per.set(0, 0);
			SmoothPeriod.set(0, 0);

		} else {


			double p0 = (Chart.High.get(0) + Chart.Low.get(0)) / 2;
			double p1 = (Chart.High.get(1) + Chart.Low.get(1)) / 2;
			double p2 = (Chart.High.get(2) + Chart.Low.get(2)) / 2;
			double p3 = (Chart.High.get(3) + Chart.Low.get(3)) / 2;  

			InPhase.set(0,p0); 
			Quadrature.set(0,p0);	

			Smooth.set(0, (4 * p0 + 3 * p1 + 2 * p2 + p3) / 10);

			double detrender = (0.0962 * Smooth.get(0) + 0.5769 * Smooth.get(2) - 0.5769 * Smooth.get(4) - 0.0962 * Smooth.get(6)) * (0.075  + 0.54);
			Detrender.set(0,detrender);

			double q1 = (0.0962 * Detrender.get(0) + 0.5769 * Detrender.get(2)  -  0.5769 * Detrender.get(4) - 0.0962 * Detrender.get(6)) * (0.075  + 0.54);	
			Q1.set(0,q1); 
			I1.set(0,Detrender.get(3)); 

			double ji = (0.0962 * I1.get(0) + 0.5769 * I1.get(2) - 0.5769 * I1.get(4) - 0.0962 * I1.get(6)) * (0.075  + 0.54); 	
			jI.set(0,ji);
			
			double jq = (0.0962 * Q1.get(0) + 0.5769 * Q1.get(2) - 0.5769 * Q1.get(4) - 0.0962 * Q1.get(6)) * (0.075  + 0.54); 	
			jQ.set(0,jq);		
		
			I2.set(0,I1.get(0) - jQ.get(0)); 
			Q2.set(0,Q1.get(0) + jI.get(0));
		
			Re.set(0,I2.get(0) * I2.get(0)    + Q2.get(0) * Q2.get(1));
			Im.set(0,I2.get(0) * Q2.get(1)  - Q2.get(0) * I2.get(1));

			Re.set(0,SQUtils.round(0.2   * Re.get(0)      + 0.8   * Re.get(1),7));
			Im.set(0,SQUtils.round(0.2   * Im.get(0)      + 0.8   * Im.get(1),7));

			if(Im.get(0)!=0 && Re.get(0)!=0){

				double per = 2 * Math.PI  / (Math.atan(Im.get(0) / Re.get(0)) );
		
				Per.set(0,SQUtils.round(per,4)); 

			}
			else Per.set(0,0); 

			/*
			if(Per.get(0) > 1.5  * Per.get(1)) Per.set(0,1.5  * Per.get(1)); 			 
			if(Per.get(0) < 0.67  * Per.get(1)) Per.set(0,0.67  * Per.get(1)); 
			if(Per.get(0) < 6)  Per.set(0,6);
			if(Per.get(0) > 50)  Per.set(0,50);
			*/

			if(Per.get(0) > 1.5  * Per.get(1)) Per.set(0,SQUtils.round(1.5  * Per.get(1),4)); 			 
			if(Per.get(0) < 0.67  * Per.get(1)) Per.set(0,SQUtils.round(0.67  * Per.get(1),4)); 
			if(Per.get(0) < 6)  Per.set(0,6);
			if(Per.get(0) > 50)  Per.set(0,50);

			Per.set(0,SQUtils.round(0.2  * Per.get(0) + 0.8  * Per.get(1),4)); 
			SmoothPeriod.set(0, SQUtils.round(0.33 * Per.get(0) + 0.67 * SmoothPeriod.get(1),4)); 
			
			InPhase.set(0,I1.get(0)); 
			Quadrature.set(0,Q1.get(0));	

		}
	}
}