package SQ.Blocks.Indicators.EhlersMotherOfAdaptiveMovingAverages;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Blocks.Indicators.EhlersHilbertTransform.EhlersHilbertTransform;

import SQ.Internal.IndicatorBlock;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(MOAMA) Ehlers Mother Of Adaptive Moving Averages", display="Ehlers Mother Of Adaptive Moving Averages(@Chart@#FastLimit#,#SlowLimit#).#Line#[#Shift#]", returnType = ReturnTypes.Price)
@Help("MotherOfAdaptiveMovingAverages help text")
@Indicator(oscillator=false)
public class EhlersMotherOfAdaptiveMovingAverages extends IndicatorBlock {

	@Parameter
	public ChartData Chart;

	@Parameter(defaultValue="0.5", isPeriod=true, minValue=0, maxValue=1.5, step=0.01)
	public double FastLimit;

	@Parameter(defaultValue="0.05", isPeriod=true, minValue=0.01, maxValue=1.5, step=0.01)
	public double SlowLimit;

	@Output(name="MAMA", color=Colors.Red)
	public DataSeries MB;

	@Output(name="FAMA", color=Colors.Blue)
	public DataSeries FB;


	@Buffer
	public DataSeries Phase;


	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	/**
	 * This method is called on every bar update and here the indicator value is computed.
	 *
	 * Unlike in MT4 you don't compute indicator values for multiple bars in a loop, 
	 * you need to compute value only for the latest (current) bar.
	 * Trading engine will take care of calling this method for every bar in the chart.
	 *
	 * Actual bar for which the indicator value is computed is stored in CurrentBar variable. 
	 * If 0, it means it is the very first bar of the chart.
	 */
	@Override
	protected void OnBarUpdate() throws TradingException {
   

		if (CurrentBar < 5) {
			// what to set when there are less bars than indicator period parameter
			    MB.set(0, 0);
	  			FB.set(0, 0);
				Phase.set(0, 0);

		} else {

			EhlersHilbertTransform indicator = Indicators.EhlersHilbertTransform(Chart);

			double i1  = indicator.InPhase.get(0);
			double q1 = indicator.Quadrature.get(0);

			Phase.set(0,i1 !=0 ? Math.atan(SQUtils.round(q1/i1,6)) : 0);

			double deltaPhase = Phase.get(1) - Phase.get(0) ;
			deltaPhase = (deltaPhase < 1)? 1 : deltaPhase;

			
			double alpha = SQUtils.safeDivide(FastLimit, deltaPhase);
			alpha = (alpha < SlowLimit)? SlowLimit : alpha;

			double m =  MB.get(1);
			double f = FB.get(1);

			MB.set(0,alpha * (Chart.High.get(0) + Chart.Low.get(0)) / 2 + (1 - alpha) * m);
			FB.set(0, 0.5 * alpha * MB.get(0) + (1 - 0.5 * alpha) * f); 

		}
	}
}