package SQ.Blocks.Indicators.EntropyMath;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import java.lang.Math;

import SQ.Internal.IndicatorBlock;


@BuildingBlock(name="(EM) Entropy Math", display="Entropy Math(@Chart@#EMPeriod#)[#Shift#]", returnType = ReturnTypes.Number)
@Indicator(oscillator=true, middleValue=0, min=-0.009, max=0.009, step=0.00001)
@Help("EntropyMath help text")
public class EntropyMath extends IndicatorBlock {

	@Parameter
	public ChartData Chart;

	@Parameter(defaultValue="10", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int EMPeriod;

	@Output
	public DataSeries Value;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------


	@Override
	protected void OnBarUpdate() throws TradingException {
        double  P,G;
		double sumx=0.0;
		double sumx2= 0.0;
		double avgx = 0.0;
		double rmsx = 0.0;

		if (CurrentBar < EMPeriod) {

			Value.set(0, 0);
			
		} else {

			for(int j =0;j<EMPeriod+1;j++){

				double r = Math.log(Chart.Close.get(j)/Chart.Close.get(j+1));
				sumx+=r;
				sumx2+=r*r;				
			}

			if(EMPeriod==0) {

				avgx=Chart.Close.get(0); 
				rmsx=0.0;  
				}
			else{

				avgx=sumx/EMPeriod;
				rmsx=Math.sqrt(sumx2/EMPeriod); 
				}
			
			P = ((avgx/rmsx)+1)/2.0;
         	G = P * Math.log(1+rmsx) + (1-P) * Math.log(1-rmsx);
			
			Value.set(0,G);
		}
	}
}