package SQ.Blocks.Indicators.HurstExponent;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;

import SQ.Internal.IndicatorBlock;
import SQ.Calculators.AverageCalculator;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(HE) Hurst Exponent", display="Hurst Exponent(@Chart@#ComputedFrom#,#Period#)[#Shift#]", returnType = ReturnTypes.Number)
@Indicator(oscillator=false,step=0.01)
@Help("HurstExponent help text")
public class HurstExponent extends IndicatorBlock {

	@Parameter
	public DataSeries Input;

	@Parameter(defaultValue="24", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int Period;

	@Output
	public DataSeries Value;

	private AverageCalculator averageCalculator;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	@Override
	protected void OnInit() throws TradingException {

		averageCalculator = new AverageCalculator(AverageCalculator.SMA, Period);
	}
	@Override
	protected void OnBarUpdate() throws TradingException {
      
		if (CurrentBar < Period) {
			// what to set when there are less bars than indicator period parameter
			Value.set(0, 0);


		} else {
			double koef = 1.253314;

			averageCalculator.onBarUpdate(Input.get(0), CurrentBar);
			double mean  = averageCalculator.getValue();
			
			double sums = 0;

			double x[] = new double[Period];
			double y[] = new double[Period];
			
			int k = 0;			
			for (k = 0; k<Period;k++){
		
				x[k]= Input.get(k)-mean;
				sums+=x[k]*x[k];
			}
						
			double maxY = x[0];
      		double minY = x[0];
      		y[0] = x[0];
			
			for(k=1; k<Period; k++){
				
				y[k] = y[k-1] + x[k];
				maxY = Math.max(y[k],maxY);
				minY = Math.min(y[k],minY);
			}
						
			double iValue = 0; 
			
			if(sums   !=0){

				iValue = SQUtils.round((maxY - minY)/ (koef * Math.sqrt(sums/Period)),2);
			} 
      							 			  
			double hurst  = 0; 

			if(iValue > 0){

				hurst  = Math.log(iValue)/ Math.log(Period);
			}
				
			Value.set(0, hurst);
		}
	}
}