package SQ.Blocks.Indicators.SSL;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Calculators.AverageCalculator;

import SQ.Internal.IndicatorBlock;


@BuildingBlock(name="(SSL) SSL", display="SSL(@Chart@#LB#,#MAMethod#)[#Shift#]", returnType = ReturnTypes.Price)
@Indicator(oscillator=false)
@ParameterSet(set="LB=14,MAMethod=1")
@ParameterSet(set="LB=20,MAMethod=1")
@ParameterSet(set="LB=30,MAMethod=1")
@ParameterSet(set="LB=40,MAMethod=1")
@ParameterSet(set="LB=50,MAMethod=1")
@ParameterSet(set="LB=100,MAMethod=1")
@ParameterSet(set="LB=200,MAMethod=1")
@ParameterSet(set="LB=12,MAMethod=1")
@ParameterSet(set="LB=24,MAMethod=1")
@ParameterSet(set="LB=48,MAMethod=1")
@ParameterSet(set="LB=96,MAMethod=1")
@ParameterSet(set="LB=120,MAMethod=1")
@ParameterSet(set="LB=240,MAMethod=1")
public class SSL extends IndicatorBlock {

	@Parameter
	public ChartData Chart;

	@Parameter(defaultValue="50", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int LB;

	@Parameter(name="MAMethod", defaultValue="0")
	@Editor(type=Editors.Selection, values="Simple=0,Exponential=1,Smoothed=2,Linear weighted=3")
	public int MAMethod;

	@Output(name="SSLU", color=Colors.Yellow)
	public DataSeries SSLU;

	@Output(name="SSLD", color=Colors.Green)
	public DataSeries SSLD;

	@Buffer
	public DataSeries Hlv;


	private AverageCalculator lowAverageCalculator;
	private AverageCalculator highAverageCalculator;
	private AverageCalculator slowAverageCalculator;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	@Override
	protected void OnInit() throws TradingException {
		
		MAMethod = SQUtils.fixAllowedRange(MAMethod, 0, 3, 0);
		lowAverageCalculator = new AverageCalculator(MAMethod, LB);
		highAverageCalculator = new AverageCalculator(MAMethod, LB);
	}


	@Override
	protected void OnBarUpdate() throws TradingException {
      double indiValue;

		if (CurrentBar < LB) {
			// what to set when there are less bars than indicator period parameter
			SSLU.set(0, 0);
			SSLD.set(0, 0);
			Hlv.set(0, 0);

		} else {
			
			Hlv.set(0,Hlv.get(1));
			highAverageCalculator.onBarUpdate(Chart.High.get(0), CurrentBar);
			lowAverageCalculator.onBarUpdate(Chart.Low.get(0), CurrentBar);
	
			if(Chart.Close.get(0)>highAverageCalculator.getValue())Hlv.set(0,1); 
			
			if(Chart.Close.get(0)<lowAverageCalculator.getValue())Hlv.set(0,-1); 
			
			SSLU.set(0,lowAverageCalculator.getValue());
			
			if(Hlv.get(0)==-1){
				SSLU.set(0,lowAverageCalculator.getValue());
				SSLD.set(0,highAverageCalculator.getValue());
				
			}
			else{
				SSLU.set(0,highAverageCalculator.getValue());
				SSLD.set(0,lowAverageCalculator.getValue());

			}			
		}
	}
}