package SQ.Blocks.Indicators.WAE;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Blocks.Indicators.MACD.MACD;
import SQ.Blocks.Indicators.BollingerBands.BollingerBands;
import SQ.Blocks.Indicators.MTATR.MTATR;


import SQ.Internal.IndicatorBlock;


@BuildingBlock(name="(WAE) Waddah Attar Explosion", display="Waddah Attar Explosion(@Chart@#FastMAPeriod#,#SlowMAPeriod#,#SignalMAPeriod#,#ATRPeriod#,#BBPeriod#,#BBStdvPeriod#,#Sensetive#,#DeadZonePip#)[#Shift#]", returnType = ReturnTypes.Number)
@Indicator(oscillator=false,min=0, max=1000, step=0.001)
public class WAE extends IndicatorBlock {

	@Parameter
	public ChartData Chart;

	@Parameter(defaultValue="12", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int FastMAPeriod;

	@Parameter(defaultValue="26", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int SlowMAPeriod;

	@Parameter(defaultValue="9", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int SignalMAPeriod;

	@Parameter(defaultValue="14", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int ATRPeriod;

	@Parameter(defaultValue="20", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int BBPeriod;

	@Parameter(defaultValue="2.5", isPeriod=true, minValue=0.1, maxValue=10, step=0.1)
	public double BBStdvPeriod;

	@Parameter(defaultValue="90", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int Sensetive;

	@Parameter(defaultValue="1.5", isPeriod=true, minValue=1, maxValue=100, step=0.1)
	public double DeadZonePip;

	@Output(name="WAEPositive", color=Colors.Green)
	public DataSeries WAEPositive;

	@Output(name="WAENegative", color=Colors.Red)
	public DataSeries WAENegative;

	@Output(name="Explosion", color=Colors.Black)
	public DataSeries Explosion;

	@Output(name="Dead", color=Colors.Black)
	public DataSeries Dead;

	


	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	/**
	 * This method is called on every bar update and here the indicator value is computed.
	 *
	 * Unlike in MT4 you don't compute indicator values for multiple bars in a loop, 
	 * you need to compute value only for the latest (current) bar.
	 * Trading engine will take care of calling this method for every bar in the chart.
	 *
	 * Actual bar for which the indicator value is computed is stored in CurrentBar variable. 
	 * If 0, it means it is the very first bar of the chart.
	 */
	@Override
	protected void OnBarUpdate() throws TradingException {
      

	  int max = Math.max(FastMAPeriod,Math.max(SlowMAPeriod,Math.max(SignalMAPeriod,Math.max(SignalMAPeriod,ATRPeriod))));

		if (CurrentBar < max) {

			// what to set when there are less bars than indicator period parameter
			WAEPositive.set(0, 0);
			WAENegative.set(0, 0);
			Explosion.set(0, 0);
			Dead.set(0, 0);
		} else {
			double medianPrice = (Chart.High(0)+Chart.Low(0))/2;

			

			MACD indicator = Indicators.MACD(Chart.Median,FastMAPeriod,SlowMAPeriod,SignalMAPeriod);
  			
			double shiftMACDValue = indicator.Main.get(0);
			double shift1MACDValue = indicator.Main.get(1);
			double firstTrend = (shiftMACDValue-shift1MACDValue)*Sensetive;

			double shift2MACDValue = indicator.Main.get(2);
			double shift3MACDValue = indicator.Main.get(3);
			double secTrend = (shift2MACDValue-shift3MACDValue)*Sensetive;


			BollingerBands indicator2 = Indicators.BollingerBands(Chart.Median,BBPeriod,BBStdvPeriod);

			double curBBUpper =  indicator2.Upper.get(0);
			double curBBLower =  indicator2.Lower.get(0);
			double firstExplo = (curBBUpper-curBBLower);

			double prevBBUpper =  indicator2.Upper.get(1);
			double prevBBLower =  indicator2.Lower.get(1);
			double secExplo = (prevBBUpper-prevBBLower);


			MTATR ATRindicator = Indicators.MTATR(Chart, ATRPeriod);
			double atrValue = ATRindicator.Value.get(0);

			double dead = atrValue*DeadZonePip;

			WAEPositive.set(0, 0);
			WAENegative.set(0, 0);
			Explosion.set(0, 0);
			Dead.set(0, 0);
			
			if(firstTrend>=0){
			
				WAEPositive.set(0, firstTrend);
				WAENegative.set(0, 0);

			} 
			if(firstTrend<0){

				WAENegative.set(0, firstTrend*-1);
				WAEPositive.set(0, 0);

			} 
			
			Explosion.set(0, firstExplo);
			Dead.set(0, dead);

		}     
	}
}