package SQ.Columns.Databanks;
import java.util.ArrayList; // import the ArrayList class
import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import com.strategyquant.tradinglib.ResultsGroup;
import com.strategyquant.tradinglib.ValueTypes;
import com.strategyquant.tradinglib.results.SpecialValues;
import it.unimi.dsi.fastutil.doubles.DoubleArrayList;
import it.unimi.dsi.fastutil.ints.IntArrayList;

import java.util.*; 

import java.util.List;



public class AddMarketsRetDDMedian extends DatabankColumn {

	public AddMarketsRetDDMedian() {
		super("AddMarketsRetDDMedian", 
				DatabankColumn.Decimal2, // value display format
				ValueTypes.Maximize, // whether value should be maximized / minimized / approximated to a value   
				0, // target value if approximation was chosen  
				-1000, // average minimum of this value
				1000); // average maximum of this value

		setWidth(80); // defaultcolumn width in pixels

		setTooltip("Add.Markets Ret/DD Median");  

		/* If this new column is dependent on some other columns that have to vbe computed first, put them here.
       Make sure you don't creat circular dependency, such as A depends on B and B depends on A.
       Columns (=stats values) are identified by the name of class)
		 */
		setDependencies("ReturnDDRatio");
	}

	//------------------------------------------------------------------------


	@Override
	public double  compute(SQStats stats, StatsTypeCombination combination, OrdersList ordersList, SettingsMap settings, SQStats statsLong, SQStats statsShort, Result result, SettingsMap rgSpecialValues) throws Exception {

		if(result == null) {

			return 0;
		}

		if(!result.getResultKey().equals(ResultsGroup.Portfolio)) {

			return 0;
		}

		DoubleArrayList npList = new DoubleArrayList();
		ResultsGroup rg =  result.getResultsGroup();
		
		byte direction = combination.getDirection();
		byte plType = combination.getPLType();
		byte sampleType = combination.getSampleType();

		for (String adMarketKey : rg.additionalMarketKeys()){

			Result rs = rg.subResult(adMarketKey);
			SQStats adResultStats = rs.stats(direction, plType, sampleType);
			npList.add(adResultStats.getDouble("ReturnDDRatio"));

		}

		npList.sort(null);
		double[] tempArray = new double[npList.size()];
		npList.toArray(tempArray);
		double res = 0;
		if(tempArray.length > 2){
		fdebug(" Median Problem"," tempArray.length: "+tempArray.length);
		res = findMedian(tempArray, tempArray.length);
		}
		return res;


	}	


	private double findMedian(double a[], int n) 
	{ 
		// First we sort the array 
		Arrays.sort(a); 

		// check for even case 
		if (n % 2 != 0) 
			return (double)a[n / 2]; 

		return (double)(a[(n - 1) / 2] + a[n / 2]) / 2.0; 
	} 


}