package SQ.WhatIf;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.strategyquant.lib.SQTime;
import com.strategyquant.tradinglib.ClassConfig;
import com.strategyquant.tradinglib.Help;
import com.strategyquant.tradinglib.Order;
import com.strategyquant.tradinglib.OrdersList;
import com.strategyquant.tradinglib.Parameter;
import com.strategyquant.tradinglib.WhatIf;
import com.strategyquant.lib.SettingsMap;
import com.strategyquant.tradinglib.results.stats.comparator.OrderComparatorByOpenTime;

import it.unimi.dsi.fastutil.objects.ObjectListIterator;

@ClassConfig(name="(STIO) Stop Trading IS/OS ", display="Stop Trading IS/OS (#DDOPercTreshold#)")
@Help("Help text")
public class StopTradingTrainTest extends WhatIf {
	public static final Logger Log = LoggerFactory.getLogger(StopTradingTrainTest.class);
	

	@Parameter(name="DD Perc. Treshold", defaultValue="10", minValue=0, maxValue=1000, step=0.1)
	public double  DDOPercTreshold;

	@Override
	public void filter(OrdersList orders) throws Exception {
		

		double initialBalance = orders.get(0).AccountBalance- orders.get(0).PL;
		double accountBalance =initialBalance;
		
		double accountOSBalance = 0;
		double oSInitialAccountBalance = 0;
		double oSCurrentAccountBalance = 0;
		double oSMaxAccountBalance = 0;
		int startDeleteFromIndex = 999999999; 
		double currentOSDD = 0;
		double maxISDD = 0; 
		double maxAccountBalance = 0;
		double currentDD =0;
		
		for(int i = 0;i < orders.size();i++){

			Order order = orders.get(i); /// actual last order
				
			/// here we have to obtain max IS DD
			if(order.SampleType == 11){

				accountBalance = orders.get(i).AccountBalance;
				
				if(accountBalance>maxAccountBalance) maxAccountBalance = accountBalance; 

				currentDD =(accountBalance/maxAccountBalance-1)*100; /// actual DD after close of trade at index i 

				if(currentDD<maxISDD) maxISDD = currentDD; /// if current DD > maxISDD 
						
			}
			// We are between IS and OS part of the DATA
			if(orders.get(i).SampleType == 21 && orders.get(i-1).SampleType == 11){ 

			 	oSInitialAccountBalance =orders.get(i).AccountBalance-orders.get(i).PL; /// initial OS balance 
				oSCurrentAccountBalance = orders.get(i).AccountBalance; /// actual OS balance ( after 1. OS Trade ) 
				oSMaxAccountBalance = oSInitialAccountBalance; // oSMaxAccountBalance == oSInitialAccountBalance 
				
			 }
			/// We are in OS part of the data
			if(order.SampleType == 21){
				
				if(oSCurrentAccountBalance>oSMaxAccountBalance) oSMaxAccountBalance =oSCurrentAccountBalance;			
			
				oSCurrentAccountBalance = orders.get(i).AccountBalance;  
				currentOSDD  =(oSCurrentAccountBalance/oSMaxAccountBalance-1)*100;

				if(currentOSDD<=(DDOPercTreshold/100)*(maxISDD)){

					startDeleteFromIndex =i+1;
				
					break;
				} 						
			}		
		}

			for(ObjectListIterator<Order> d = orders.listIterator(); d.hasNext();) {
			Order toBeDeletedOrder = d.next(); 
				
			  
				if(toBeDeletedOrder.Ticket>=startDeleteFromIndex) {

					d.remove();
				}
			}
		}

}