package SQ.WhatIf;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.strategyquant.lib.SQTime;
import com.strategyquant.tradinglib.ClassConfig;
import com.strategyquant.tradinglib.Help;
import com.strategyquant.tradinglib.Order;
import com.strategyquant.tradinglib.OrdersList;
import com.strategyquant.tradinglib.Parameter;
import com.strategyquant.tradinglib.WhatIf;
import com.strategyquant.lib.SettingsMap;
import com.strategyquant.tradinglib.results.stats.comparator.OrderComparatorByOpenTime;

import it.unimi.dsi.fastutil.objects.ObjectListIterator;

@ClassConfig(name="(STTVT) Stop Trading Train/Validate/Test ", display="Stop Trading Train/Validate/Test(#DDOPercTreshold#,#Mode#)")
@Help("Help text")
public class StopTradingTrainTestValidate extends WhatIf {
	public static final Logger Log = LoggerFactory.getLogger(StopTradingTrainTestValidate.class);
	

	@Parameter(name="DD Perc Treshold", defaultValue="10", minValue=0, maxValue=1000, step=0.1)
	public double  DDOPercTreshold;

	@Parameter(name="Mode", defaultValue="1", minValue=1, maxValue=2, step=1)
	public double  Mode;


	@Override
	public void filter(OrdersList orders) throws Exception {
		

		double initialBalance = orders.get(0).AccountBalance- orders.get(0).PL;
		double accountBalance =initialBalance;
		double accountOSBalance = 0;
		
		double testInitialAccountBalance = 0;
		double testCurrentAccountBalance = 0;
		double testMaxAccountBalance = 0;
		
		double validateInitialAccountBalance = 0;
		double validateCurrentAccountBalance = 0;
		double validateMaxAccountBalance = 0;
		
		int startTestDeleteFromIndex= 999999999; 
		int startValidateDeleteFromIndex= 999999999; 
		int endTestDeleteFromIndex=0; 
		int endValidateDeleteFromIndex=0;
		
		double currentTestDD = 0;
		double currentValidateDD = 0;




		double maxISDD = 0; /// pozitivna hodnota treba brat v uvahu dalej alebo minusovat
		double maxAccountBalance = 0;
		double currentDD =0;
		for(int i = 0;i < orders.size();i++){


			Order order = orders.get(i); /// aktualny posledny order o ktorom sa rozhodujeme ci pojde alebo nepojde do newOL 

			// TRAIN DATA
			if(order.SampleType == 11){
				
				accountBalance = orders.get(i).AccountBalance;
				
				if(accountBalance>maxAccountBalance) maxAccountBalance = accountBalance; 

				currentDD =(accountBalance/maxAccountBalance-1)*100; /// actual DD after close of trade at index i 

				if(currentDD<maxISDD) maxISDD = currentDD; /// if current DD > maxISDD 
						
			}	
			
			// We are between TRAIN/TEST
			if(orders.get(i).SampleType == 41 && orders.get(i-1).SampleType == 11){ 
					
			 	testInitialAccountBalance =orders.get(i).AccountBalance-orders.get(i).PL; /// init OS balance 
				testCurrentAccountBalance = orders.get(i).AccountBalance; /// actual OS balance ( after 1. OS Trade ) 
				testMaxAccountBalance = testInitialAccountBalance; // testMaxAccountBalance == testInitialAccountBalance 
				
			}
			 	
			// TEST PART
			if(order.SampleType == 41){


				if(testCurrentAccountBalance>testMaxAccountBalance) testMaxAccountBalance =testCurrentAccountBalance;	 		
			
				testCurrentAccountBalance = orders.get(i).AccountBalance;  /// 

				currentTestDD  =(testCurrentAccountBalance/testMaxAccountBalance-1)*100;
				
				if(Mode==2) maxISDD =currentTestDD;
					
				if(Mode ==1){
				/// zistenie kedy treba mazat ordre	
					if(currentTestDD<=(DDOPercTreshold/100)*(maxISDD)){

						if(startTestDeleteFromIndex == 999999999) startTestDeleteFromIndex =i+1; // you need to set variable with index

					}
				}							
			}
		
			if(orders.get(i).SampleType == 21 && orders.get(i-1).SampleType == 41){ 
				
				endTestDeleteFromIndex = i;

			 	validateInitialAccountBalance = orders.get(i).AccountBalance-orders.get(i).PL; /// init OS balance 
				validateCurrentAccountBalance = orders.get(i).AccountBalance; /// actual OS balance ( after 1. OS Trade ) 
				validateMaxAccountBalance = validateInitialAccountBalance; // validateMaxAccountBalance == validateInitialAccountBalance 

			}
				
			if(order.SampleType == 21){

		
				if(validateCurrentAccountBalance>validateMaxAccountBalance) validateMaxAccountBalance =validateCurrentAccountBalance;;

				validateCurrentAccountBalance = orders.get(i).AccountBalance;  /// 
				currentValidateDD  =(validateCurrentAccountBalance/validateMaxAccountBalance-1)*100;

				if(currentValidateDD<=(DDOPercTreshold/100)*(maxISDD)){
								
					if(startValidateDeleteFromIndex==999999999) startValidateDeleteFromIndex =i+1;
					break;
				} 			
			}
		}
	
	
		for(ObjectListIterator<Order> d = orders.listIterator(); d.hasNext();) {
	
			Order toBeDeletedOrder = d.next(); 
				
				if(toBeDeletedOrder.Ticket>=startTestDeleteFromIndex && toBeDeletedOrder.Ticket <=endTestDeleteFromIndex) {

					d.remove();
				}

				if(toBeDeletedOrder.Ticket>=startValidateDeleteFromIndex) {

					d.remove();
				}
			}				
		}	
}
